@@ -1515,10 +1515,11 @@ public class MainWindow {
 
 	public static void HandleHelp (object sender, EventArgs args)
 	{
-		Gnome.Vfs.Url.Show("ghelp:" + 
-				FSpot.Global.HelpDirectory +
-				System.IO.Path.DirectorySeparatorChar +
-				"index.xml");
+		GnomeUtil.ShowHelp ("f-spot.xml", null, Toplevel.Window.Screen, null);
+//		Gnome.Vfs.Url.Show("ghelp:" + 
+//				FSpot.Global.HelpDirectory +
+//				System.IO.Path.DirectorySeparatorChar +
+//				"f-spot.xml");
 	}
 
 	public static void HandleAbout (object sender, EventArgs args)
@@ -1568,7 +1569,7 @@ public class MainWindow {
 
                 new About ("F-Spot", 
 			   FSpot.Defines.VERSION, 
-			   Catalog.GetString ("Copyright \x00a9 2003-2006 Novell Inc."),
+			   Catalog.GetString ("Copyright \x00a9 2003-2007 Novell Inc."),
                            null, authors, new string [0], translators, 
 			   PixbufUtils.LoadFromAssembly("f-spot-logo-tango.svg")).Show();
 	}
@@ -1645,7 +1646,7 @@ public class MainWindow {
 		Close();
 	}
 	
-	void Close ()
+	public void Close ()
 	{
 		int x, y, width, height;
 		main_window.GetPosition (out x, out y);
