@@ -448,7 +448,7 @@ public class PhotoView : EventBox {
 		description_delay = new FSpot.Delay (1000, new GLib.IdleHandler (CommitPendingChanges));
 		this.Destroyed += HandleDestroy;
 
-
+		Name = "ImageContainer";
 		Box vbox = new VBox (false, 6);
 		Add (vbox);
 
@@ -524,6 +524,15 @@ public class PhotoView : EventBox {
 		toolbar_hbox.PackStart (sepia_button, false, true, 0);
 		sepia_button.Clicked += HandleSepiaButtonClicked;
 
+		ItemAction straighten = new TiltEditorAction (photo_view);
+		toolbar_hbox.PackStart (straighten.GetToolButton (false), false, true, 0);
+		
+		ItemAction softfocus = new SoftFocusEditorAction (photo_view);
+		toolbar_hbox.PackStart (softfocus.GetToolButton (false), false, true, 0);
+
+		ItemAction autocolor = new AutoColor (photo_view.Item);
+		toolbar_hbox.PackStart (autocolor.GetToolButton (false), false, true, 0);
+
 		/* Spacer Label */
 		toolbar_hbox.PackStart (new Label (""), true, true, 0);
 
