@@ -16,6 +16,8 @@ using System.Collections.Specialized;
 using System.Web;
 using Mono.Unix;
 
+using FSpot.Filters;
+
 using Gnome.Keyring;
 
 using Mono.Google;
@@ -26,22 +28,37 @@ namespace FSpot {
 
 		private string username;
 		private string password;
-		private bool connected;
+		private string token;
+		private string unlock_captcha;
 		private GoogleConnection connection;
 		private PicasaWeb picasa;
 
-		public GoogleAccount (string username, string password)
+		public GoogleAccount (string username, string password) 
 		{
 			this.username = username;
 			this.password = password;
 		}
 
+		public GoogleAccount (string username, string password, string token, string unlock_captcha)
+		{
+			this.username = username;
+			this.password = password;
+			this.token = token;
+			this.unlock_captcha = unlock_captcha;
+		}
+
 		public PicasaWeb Connect ()
 		{
 			System.Console.WriteLine ("GoogleAccount.Connect()");
 			GoogleConnection conn = new GoogleConnection (GoogleService.Picasa);
 			ServicePointManager.CertificatePolicy = new NoCheckCertificatePolicy ();
-			conn.Authenticate(username, password);
+			if (unlock_captcha == null || token == null)
+				conn.Authenticate(username, password);
+			else {
+				conn.Authenticate(username, password, token, unlock_captcha);
+				token = null; 
+				unlock_captcha = null;
+			}
 			connection = conn;
 			PicasaWeb picasa = new PicasaWeb(conn);
 			this.picasa = picasa;
@@ -51,7 +68,6 @@ namespace FSpot {
 		private void MarkChanged()
 		{
 			connection = null;
-			connected = false;
 		}
 
 		public bool Connected {
@@ -84,6 +100,24 @@ namespace FSpot {
 			}
 		}
 
+		public string Token {
+			get {
+				return token;
+			}
+			set {
+				token = value;
+			}
+		}
+
+		public string UnlockCaptcha {
+			get {
+				return unlock_captcha;
+			}
+			set {
+				unlock_captcha = value;
+			}
+		}
+
 		public PicasaWeb Picasa {
 			get {
 				return picasa;
@@ -206,12 +240,12 @@ namespace FSpot {
 	}
 	
 	public class GoogleAccountDialog : GladeDialog {
-		public GoogleAccountDialog (Gtk.Window parent) : this (parent, null, false) { 
+		public GoogleAccountDialog (Gtk.Window parent) : this (parent, null, false, null) { 
 			Dialog.Response += HandleAddResponse;
 			add_button.Sensitive = false;
 		}
 		
-		public GoogleAccountDialog (Gtk.Window parent, GoogleAccount account, bool show_error) :  base ("google_add_dialog")
+		public GoogleAccountDialog (Gtk.Window parent, GoogleAccount account, bool show_error, CaptchaException captcha_exception) :  base ("google_add_dialog")
 		{
 			this.Dialog.Modal = false;
 			this.Dialog.TransientFor = parent;
@@ -219,7 +253,23 @@ namespace FSpot {
 			
 			this.account = account;
 
+			bool show_captcha = (captcha_exception != null);
 			status_area.Visible = show_error;
+			locked_area.Visible = show_captcha;
+			captcha_label.Visible = show_captcha;
+			captcha_entry.Visible = show_captcha;
+			captcha_image.Visible = show_captcha;
+			
+			password_entry.ActivatesDefault = true;
+			username_entry.ActivatesDefault = true;
+
+			if (show_captcha) {
+				try {
+					ImageFile img = ImageFile.Create(new Uri(captcha_exception.CaptchaUrl));
+					captcha_image.Pixbuf = img.Load();
+					token = captcha_exception.Token;
+				} catch (Exception) {}
+			}
 
 			if (account != null) {
 				password_entry.Text = account.Password;
@@ -243,11 +293,7 @@ namespace FSpot {
 			password = password_entry.Text;
 			username = username_entry.Text;
 
-			if (password == "" || username == "")
-				add_button.Sensitive = false;
-			else
-				add_button.Sensitive = true;
-
+			add_button.Sensitive = !(password == "" || username == "");
 		}
 		
 		[GLib.ConnectBefore]
@@ -265,6 +311,8 @@ namespace FSpot {
 			if (args.ResponseId == Gtk.ResponseType.Ok) {
 				account.Username = username;
 				account.Password = password;
+				account.Token = token;
+				account.UnlockCaptcha = captcha_entry.Text;
 				GoogleAccountManager.GetInstance ().MarkChanged (true, account);
 			} else if (args.ResponseId == Gtk.ResponseType.Reject) {
 				// NOTE we are using Reject to signal the remove action.
@@ -276,16 +324,24 @@ namespace FSpot {
 		private GoogleAccount account;
 		private string password;
 		private string username;
+		private string token;
+
 
 		// widgets 
 		[Glade.Widget] Gtk.Entry password_entry;
 		[Glade.Widget] Gtk.Entry username_entry;
+		[Glade.Widget] Gtk.Entry captcha_entry;
 
 		[Glade.Widget] Gtk.Button add_button;
 		[Glade.Widget] Gtk.Button remove_button;
 		[Glade.Widget] Gtk.Button cancel_button;
 
 		[Glade.Widget] Gtk.HBox status_area;
+		[Glade.Widget] Gtk.HBox locked_area;
+
+		[Glade.Widget] Gtk.Image captcha_image;
+		[Glade.Widget] Gtk.Label captcha_label;
+
 	}
 
 	public class GoogleAddAlbum : GladeDialog {
@@ -348,7 +404,7 @@ namespace FSpot {
 	public class GoogleExport : GladeDialog {
 		public GoogleExport (IBrowsableCollection selection) : base ("google_export_dialog")
 		{
-			this.photos = (Photo []) selection.Items;
+			this.items = selection.Items;
 			album_button.Sensitive = false;
 			IconView view = new IconView (selection);
 			view.DisplayDates = false;
@@ -369,7 +425,6 @@ namespace FSpot {
 			if (edit_button != null)
 				edit_button.Clicked += HandleEditGallery;
 			
-			rh = new Gtk.ResponseHandler (HandleResponse);
 			Dialog.Response += HandleResponse;
 			connect = true;
 			HandleSizeActive (null, null);
@@ -384,8 +439,6 @@ namespace FSpot {
 //			LoadPreference (Preferences.EXPORT_PICASAWEB_META);
 		}
 		
-		Gtk.ResponseHandler rh;
-		
 		private bool scale;
 		private int size;
 		private bool browser;
@@ -393,10 +446,10 @@ namespace FSpot {
 //		private bool meta;
 		private bool connect = false;
 
-		private long exported_size = 0;
+		private long approx_size = 0;
 		private long sent_bytes = 0;
 
-		Photo [] photos;
+		IBrowsableItem [] items;
 		int photo_index;
 		FSpot.ThreadProgressDialog progress_dialog;
 		
@@ -406,10 +459,6 @@ namespace FSpot {
 
 		private string xml_path;
 
-		// Dialogs
-		private GoogleAccountDialog gallery_add;
-		private GoogleAddAlbum album_add;
-
 		// Widgets
 		[Glade.Widget] Gtk.OptionMenu gallery_optionmenu;
 		[Glade.Widget] Gtk.OptionMenu album_optionmenu;
@@ -463,9 +512,9 @@ namespace FSpot {
 				Dialog.Destroy ();
 
 				command_thread = new System.Threading.Thread (new System.Threading.ThreadStart (this.Upload));
-				command_thread.Name = Mono.Posix.Catalog.GetString ("Uploading Pictures");
+				command_thread.Name = Catalog.GetString ("Uploading Pictures");
 				
-				progress_dialog = new FSpot.ThreadProgressDialog (command_thread, photos.Length);
+				progress_dialog = new FSpot.ThreadProgressDialog (command_thread, items.Length);
 				progress_dialog.Start ();
 
 				// Save these settings for next time
@@ -484,97 +533,91 @@ namespace FSpot {
 
 		private void HandleUploadProgress(object o, UploadProgressEventArgs args)
 		{
-			if (!scale) {
-				progress_dialog.ProgressText = System.String.Format ("{0} of {1}",ToHumanReadableSize(sent_bytes + args.BytesSent), ToHumanReadableSize(exported_size));	
-				progress_dialog.Fraction = (sent_bytes + args.BytesSent) / (double)exported_size;
-
-			} else {
-				if (exported_size == 0)
-					progress_dialog.ProgressText = System.String.Format ("{0} Sent",ToHumanReadableSize(args.BytesSent));	
+				if (approx_size == 0)
+					progress_dialog.ProgressText = System.String.Format ("{0} Sent",SizeUtil.ToHumanReadable(args.BytesSent));
 				else
-					progress_dialog.ProgressText = System.String.Format ("{0} of approx. {1}", ToHumanReadableSize(sent_bytes + args.BytesSent), ToHumanReadableSize(exported_size));
-				progress_dialog.Fraction = ((photo_index - 1) / (double) photos.Length) + (args.BytesSent / (args.BytesTotal * (double) photos.Length));
-			}
+					progress_dialog.ProgressText = System.String.Format ("{0} of approx. {1}", SizeUtil.ToHumanReadable(sent_bytes + args.BytesSent), SizeUtil.ToHumanReadable(approx_size));
+				progress_dialog.Fraction = ((photo_index - 1) / (double) items.Length) + (args.BytesSent / (args.BytesTotal * (double) items.Length));
 		}
 
 		private void Upload ()
 		{
 			try {
 				album.UploadProgress += HandleUploadProgress;
+				sent_bytes = 0;
+				approx_size = 0;
 
 				System.Console.WriteLine ("Starting Upload to Picasa");
 
-				if (!scale)
-					foreach (Photo photo in photos) {
-						System.IO.FileInfo file_info = new System.IO.FileInfo(photo.GetVersionPath(photo.DefaultVersionId));
-						exported_size += file_info.Length;
-					}
-					
+				FilterSet filters = new FilterSet ();
+				filters.Add (new JpegFilter ());
 
-				while (photo_index < photos.Length) {
-					Photo photo = photos [photo_index];
+				if (scale)
+					filters.Add (new ResizeFilter ((uint)size));
 
-					System.IO.FileInfo file_info;
-					System.Console.WriteLine ("uploading {0}", photo_index);
+				if (rotate)
+					filters.Add (new OrientationFilter ());
 
-					progress_dialog.Message = System.String.Format (Mono.Posix.Catalog.GetString ("Uploading picture \"{0}\" ({1} of {2})"), photo.Name, photo_index+1, photos.Length);
+				while (photo_index < items.Length) {
+					IBrowsableItem item = items[photo_index];
+
+					FileInfo file_info;
+					Console.WriteLine ("uploading {0}", photo_index);
+
+					progress_dialog.Message = String.Format (Catalog.GetString ("Uploading picture \"{0}\" ({1} of {2})"), 
+										 item.Name, photo_index+1, items.Length);
 					photo_index++;
-
 					
-					string orig = photo.DefaultVersionUri.LocalPath;
-					string path = PixbufUtils.Resize (orig, size, true);
-					string final = path + System.IO.Path.GetExtension (orig);
-					if (scale) {
-						orig = photo.DefaultVersionUri.LocalPath;
-						path = PixbufUtils.Resize (orig, size, true);
-						final = path + System.IO.Path.GetExtension (orig);
-						System.IO.File.Move (path, final);
-						if (photo_index == 1) {
-							file_info = new System.IO.FileInfo(final);
-							exported_size = photos.Length * file_info.Length;
-							Console.WriteLine ("total size: {0}", exported_size);
+					FilterRequest request = new FilterRequest (item.DefaultVersionUri);
+
+					filters.Convert (request);
+
+					file_info = new FileInfo (request.Current.LocalPath);
+
+					if (approx_size == 0) //first image
+						approx_size = file_info.Length * items.Length;
+					else
+						approx_size = sent_bytes * items.Length / (photo_index - 1);
+
+					string id = album.UploadPicture (request.Current.LocalPath, item.Description);
+					if (Core.Database != null) {
+						System.UriBuilder uri = new System.UriBuilder (album.Link);
+						uri.Path = uri.Path + "/photo";
+						//Setting the Fragment doesn't display it :( workaround follows
+						string photouri;
+						if (uri.Query == null || uri.Query == String.Empty) {
+							uri.Fragment = id;
+							photouri = uri.ToString () + uri.Fragment;
+						} else {
+							uri.Query = uri.Query.Substring (1) + "#" + id;
+							photouri = uri.ToString ();
 						}
-
-						file_info = new System.IO.FileInfo(final);
-						sent_bytes += file_info.Length;
-						album.UploadPicture (final);
-						System.IO.File.Delete (final);
-					} else if (rotate){
-						orig = photo.DefaultVersionUri.LocalPath;
-						path = ImageFile.TempPath (orig);
-
-						if (OrientationFilter.Convert (orig, path))
-							album.UploadPicture (final, photo.Description);
-						else 
-							album.UploadPicture (photo.DefaultVersionUri.LocalPath, photo.Description);
-							
-						System.IO.File.Delete (final);
-					} else {
-						album.UploadPicture (photo.DefaultVersionUri.LocalPath, photo.Description);
+						Core.Database.Exports.Create ((item as Photo).Id,
+									      (item as Photo).DefaultVersionId,
+									      ExportStore.PicasaExportType,
+									      photouri);
 					}
 
+					sent_bytes += file_info.Length;
 
-					if (!scale) {
-						file_info = new System.IO.FileInfo(photo.GetVersionPath(photo.DefaultVersionId));
-						sent_bytes += file_info.Length;
-					}
-						
+					request.Dispose ();
 				}
-
-				progress_dialog.Message = Mono.Posix.Catalog.GetString ("Done Sending Photos");
+				
+				progress_dialog.Message = Catalog.GetString ("Done Sending Photos");
 				progress_dialog.Fraction = 1.0;
-				progress_dialog.ProgressText = Mono.Posix.Catalog.GetString ("Upload Complete");
+				progress_dialog.ProgressText = Catalog.GetString ("Upload Complete");
 				progress_dialog.ButtonLabel = Gtk.Stock.Ok;
+
+				if (browser) {
+					GnomeUtil.UrlShow (null, album.Link);
+				}
 			} catch (System.Exception e) {
-				progress_dialog.Message = String.Format (Mono.Posix.Catalog.GetString ("Error Uploading To Gallery: {0}"),
+				progress_dialog.Message = String.Format (Catalog.GetString ("Error Uploading To Gallery: {0}"),
 									 e.Message);
-				progress_dialog.ProgressText = Mono.Posix.Catalog.GetString ("Error");
+				progress_dialog.ProgressText = Catalog.GetString ("Error");
 				System.Console.WriteLine (e);
 			}
 			
-			if (browser) {
-				GnomeUtil.UrlShow (null, album.Link);
-			}
 		}
 
 		private void HandleScaleCheckToggled (object o, EventArgs e)
@@ -590,7 +633,7 @@ namespace FSpot {
 
 			accounts = manager.GetAccounts ();
 			if (accounts == null || accounts.Count == 0) {
-				Gtk.MenuItem item = new Gtk.MenuItem (Mono.Posix.Catalog.GetString ("(No Gallery)"));
+				Gtk.MenuItem item = new Gtk.MenuItem (Catalog.GetString ("(No Gallery)"));
 				menu.Append (item);
 				gallery_optionmenu.Sensitive = false;
 				edit_button.Sensitive = false;
@@ -613,37 +656,17 @@ namespace FSpot {
 			gallery_optionmenu.SetHistory ((uint)pos);
 		}
 
-		private static string ToHumanReadableSize (long in_size)
-		{
-			string tmp_str = "";
-			float tmp_size = in_size;
-			int k = 0;
-			string[] size_abr = {"bytes", "kB", "MB", "GB", "TB" };
-			
-			while (tmp_size > 700) { //it's easier to read 0.9MB than 932kB
-				tmp_size = tmp_size / 1024;
-				k++;
-			}
-			
-			if (tmp_size < 7)
-				tmp_str = tmp_size.ToString ("#.##");
-			else if (tmp_size < 70)
-				tmp_str = tmp_size.ToString ("##.#");
-			else
-				tmp_str = tmp_size.ToString ("#,###");
-			
-			if (k < size_abr.Length)
-				return tmp_str + " " + size_abr[k];
-			else
-				return in_size.ToString();
-		}
-	
 		private void Connect ()
 		{
 			Connect (null);
 		}
 
 		private void Connect (GoogleAccount selected)
+		{
+			Connect (selected, null, null);
+		}
+
+		private void Connect (GoogleAccount selected, string token, string text)
 		{
 			try {
 				if (accounts.Count != 0 && connect) {
@@ -662,11 +685,11 @@ namespace FSpot {
 
 					StringBuilder sb = new StringBuilder("<small>");
 					sb.Append(Catalog.GetString("Available space :"));
-					sb.Append(ToHumanReadableSize(ql - qu));
+					sb.Append(SizeUtil.ToHumanReadable (ql - qu));
 					sb.Append(" (");
 					sb.Append(100 * qu / ql);
 					sb.Append("% used out of ");
-					sb.Append(ToHumanReadableSize(ql));
+					sb.Append(SizeUtil.ToHumanReadable (ql));
 					sb.Append(")");
 					sb.Append("</small>");
 
@@ -675,23 +698,30 @@ namespace FSpot {
 
 					album_button.Sensitive = true;
 				}
-			} catch (System.Exception ex) {
+			} catch (CaptchaException exc){
+				System.Console.WriteLine("Your google account is locked");
 				if (selected != null)
 					account = selected;
 
-				System.Console.WriteLine("Can not connect to Picasa. Bad username ? password ? network connection ?");
+				PopulateAlbumOptionMenu (account.Picasa);
+				album_button.Sensitive = false;
+
+				new GoogleAccountDialog (this.Dialog, account, false, exc);
+				
+				System.Console.WriteLine ("Your google account is locked, you can unlock it by visiting: {0}", CaptchaException.UnlockCaptchaURL);
+
+			} catch (System.Exception) {
+				System.Console.WriteLine ("Can not connect to Picasa. Bad username ? password ? network connection ?");
 				//System.Console.WriteLine ("{0}",ex);
+				if (selected != null)
+					account = selected;
+
 				PopulateAlbumOptionMenu (account.Picasa);
 
 				status_label.Text = "";
 				album_button.Sensitive = false;
 				
-				GoogleAccountDialog dialog = new GoogleAccountDialog (this.Dialog, account, true);
-				Gtk.ResponseType response = (Gtk.ResponseType) dialog.Dialog.Run ();
-
-				if (response == Gtk.ResponseType.Ok) {
-					Connect (account);					
-				}
+				new GoogleAccountDialog (this.Dialog, account, true, null);
 			} 
 		}
 
@@ -717,8 +747,12 @@ namespace FSpot {
 		{
 			PicasaAlbumCollection albums = null;
 			if (picasa != null) {
-				//gallery.FetchAlbumsPrune ();
-				albums = picasa.GetAlbums();
+				try {
+					albums = picasa.GetAlbums();
+				} catch {
+					Console.WriteLine("Can't get the albums");
+					picasa = null;
+				}
 			}
 
 			Gtk.Menu menu = new Gtk.Menu ();
@@ -726,8 +760,8 @@ namespace FSpot {
 			bool disconnected = picasa == null || !account.Connected || albums == null;
 
 			if (disconnected || albums.Count == 0) {
-				string msg = disconnected ? Mono.Posix.Catalog.GetString ("(Not Connected)") 
-					: Mono.Posix.Catalog.GetString ("(No Albums)");
+				string msg = disconnected ? Catalog.GetString ("(Not Connected)") 
+					: Catalog.GetString ("(No Albums)");
 
 				Gtk.MenuItem item = new Gtk.MenuItem (msg);
 				menu.Append (item);
@@ -742,22 +776,14 @@ namespace FSpot {
 				foreach (PicasaAlbum album in albums.AllValues) {
 					System.Text.StringBuilder label_builder = new System.Text.StringBuilder ();
 					
-//					for (int i=0; i < album.Parents.Count; i++) {
-//						label_builder.Append ("  ");
-//					}
 					label_builder.Append (album.Title);
 
 					Gtk.MenuItem item = new Gtk.MenuItem (label_builder.ToString ());
 					((Gtk.Label)item.Child).UseUnderline = false;
 					menu.Append (item);
-			
-				        //AlbumPermission add_permission = album.Perms & AlbumPermission.Add;
-
-//					if (add_permission == 0)
-//						item.Sensitive = false;
 				}
 
-				ok_button.Sensitive = photos.Length > 0;
+				ok_button.Sensitive = items.Length > 0;
 				album_optionmenu.Sensitive = true;
 				album_button.Sensitive = true;
 			}
@@ -768,12 +794,12 @@ namespace FSpot {
 		
 		public void HandleAddGallery (object sender, System.EventArgs args)
 		{
-			gallery_add = new GoogleAccountDialog (this.Dialog);
+			new GoogleAccountDialog (this.Dialog);
 		}
 		
 		public void HandleEditGallery (object sender, System.EventArgs args)
 		{
-			gallery_add = new GoogleAccountDialog (this.Dialog, account, false);
+			new GoogleAccountDialog (this.Dialog, account, false, null);
 		}
 
 		public void HandleAddAlbum (object sender, System.EventArgs args)
@@ -781,7 +807,7 @@ namespace FSpot {
 			if (account == null)
 				throw new Exception (Catalog.GetString ("No account selected"));
 				
-			album_add = new GoogleAddAlbum (this, account.Picasa);
+			new GoogleAddAlbum (this, account.Picasa);
 		}
 
 		void LoadPreference (string key)
