@@ -72,9 +72,9 @@ public class PixbufLoader {
 	public PixbufLoader ()
 	{
 		queue = new ArrayList ();
-		requests_by_path = new Hashtable ();
+		requests_by_path = Hashtable.Synchronized (new Hashtable ());
 		processed_requests = new Queue ();
-
+		
 		pending_notify = new ThreadNotify (new Gtk.ReadyEvent (HandleProcessedRequests));
 
 		worker_thread = new Thread (new ThreadStart (WorkerThread));
@@ -118,6 +118,16 @@ public class PixbufLoader {
 		}
 	}
 
+	public void Request (Uri uri, int order)
+	{
+		Request (uri.ToString (), order);
+	}
+
+	public void Request (Uri uri, int order, int width, int height)
+	{
+		Request (uri.ToString (), order, width, height);
+	}
+
 	public void Cancel (string path)
 	{
 		lock (queue) {
@@ -135,15 +145,21 @@ public class PixbufLoader {
 	{
 		Pixbuf orig_image;
 		try {
-			FSpot.ImageFile img = FSpot.ImageFile.Create (request.path);
+			FSpot.ImageFile img;
+
+			if (System.IO.File.Exists (request.path))
+				img = FSpot.ImageFile.Create (request.path);
+		        else
+				img = FSpot.ImageFile.Create (new Uri (request.path));
+
 			if (request.width > 0) {
 				orig_image = img.Load (request.width, request.height);
 			} else
 				orig_image = img.Load ();
 
 		} catch (GLib.GException e){
-								System.Console.WriteLine (e.ToString ());
-return;		
+			System.Console.WriteLine (e.ToString ());
+			return;		
 		}
 		
 		if (orig_image == null)
@@ -196,9 +212,9 @@ return;
 						pending_notify.WakeupMain ();
 						pending_notify_notified = true;
 					}
-						
+					
 					current_request = null;
-					}
+				}
 			}
 
 			lock (queue) {
@@ -228,7 +244,8 @@ return;
 	private void HandleProcessedRequests ()
 	{
 		Queue results;
-
+		
+		
 		lock (processed_requests) {
 			/* Copy the queued items out of the shared queue so we hold the lock for
 			   as little time as possible.  */
